/*
 * Interrupt attach
 *
 * (c) 2005, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#include <sys/neutrino.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/syspage.h>
#include <inttypes.h>


struct sigevent         int_event;
int                     id          = -1;
int                     interrupt_n = -1;
int                     mask_flag   = 0;
unsigned                period_ns   = 0;
volatile unsigned long  int_counter = 0;
volatile uint64_t       prev_nsec   = 0;


const struct sigevent *handler( void *area, int id )
{
    if ( mask_flag )
        InterruptMask( interrupt_n, id );

    int_counter++;

    if ( !period_ns )
        return (&int_event);
    else {
        if ( (SYSPAGE_ENTRY( qtime )->nsec - prev_nsec) > period_ns )
        {
            prev_nsec = SYSPAGE_ENTRY( qtime )->nsec;
            return (&int_event);
        } else
            return (NULL);
    }
}


int main( int argc, char **argv )
{
    int         opt;

    //  Allow IO operations
    if ( ThreadCtl( _NTO_TCTL_IO, 0 ) != EOK )
    {
        printf( "You must be root!" );
        return (-1);
    }

    printf( "Interrupt attach program \n" );

    while ( (opt = getopt( argc, argv, "i:p:m" )) != -1 )
    {
        switch ( opt )
        {
            case 'i':
                interrupt_n = strtoul( optarg, NULL, 0 );
                break;

            case 'p':
                period_ns  = strtoul( optarg, NULL, 0 );
                period_ns *= 1000 * 1000; // nanoseconds
                break;

            case 'm':
                mask_flag = 1;
                break;

            default:
                printf( "Unsupported option.\n" );
                return (-1);
        }
    }

    if( interrupt_n < 0 )
    {
        printf( "Incorrect interrupt number %d \n", interrupt_n );
        return (-1);
    } else
        printf( "Attaching handler for interrupt %u ( 0x%X ) \n\n", interrupt_n, interrupt_n );

    int_event.sigev_notify = SIGEV_INTR;

    // Attach interrupt handler
    id = InterruptAttach( interrupt_n, &handler, NULL, 0, 0 );

    if ( id == -1 )
    {
        perror( "Cannot attach interrupt handler" );
        return (-1);
    }

    if ( period_ns )
    {
        printf( "Counting number of interrupts %u ( 0x%X ) per %u milliseconds \n", interrupt_n, interrupt_n, period_ns / 1000000 );
        int_counter = 0;
        prev_nsec   = SYSPAGE_ENTRY( qtime )->nsec;
    }

    while ( 1 )
    {
        InterruptWait( 0, NULL );

        if ( int_counter )
            printf( "Interrupt %u ( 0x%X ) : %8ld \n", interrupt_n, interrupt_n, int_counter );

        if ( period_ns )
            int_counter = 0;

        if ( mask_flag )
            InterruptUnmask( interrupt_n, id );
    }

    InterruptDetach( id );

    return (0);
}
