/*
 * SPI test
 *
 * (c) 2019, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/resmgr.h>
#include <sys/neutrino.h>
#include <hw/inout.h>
#include <hw/nicinfo.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <inttypes.h>
#include <string.h>
#include <errno.h>
#include <devctl.h>
#include <hw/spi-master.h>

#include "spim_test.h"


typedef int(*p0FuncPtrType)();
typedef int(*p1FuncPtrType)( int p1 );
typedef int(*p2FuncPtrType)( int p1, int p2 );
typedef int(*p3FuncPtrType)( int p1, int p2, int p3 );
typedef int(*p4FuncPtrType)( int p1, int p2, int p3, int p4 );
typedef union {
    p0FuncPtrType   p0FuncPtr;
    p1FuncPtrType   p1FuncPtr;
    p2FuncPtrType   p2FuncPtr;
    p3FuncPtrType   p3FuncPtr;
    p4FuncPtrType   p4FuncPtr;
} funcPtrUnion;
typedef struct {
    char            *cmd_name;
    int             num_params;
    int             display_output;
    funcPtrUnion    funcPtr;
    char            *help;
} commandEntryType;


commandEntryType commandEntries[] = {
    // name, # params, output, function
    { "devinfo", 1, 0, (funcPtrUnion)tspi_devinfo, "<dev_index: 0='/dev/spi0' / 1='/dev/spi1'>" },
    { "drvinfo", 1, 0, (funcPtrUnion)tspi_drvinfo, "<dev_index: 0='/dev/spi0' / 1='/dev/spi1'>" },
    { "setcfg",  4, 0, (funcPtrUnion)tspi_setcfg,  "<dev_index: 0='/dev/spi0' / 1='/dev/spi1'> <cs: 0..3> <mode> <clock_rate>" },
    { "all",     2, 0, (funcPtrUnion)tspi_testall, "<dev_index: 0='/dev/spi0' / 1='/dev/spi1'> <cs: 0..3>" },
    { "io",      3, 0, (funcPtrUnion)tspi_io_test, "<dev_index: 0='/dev/spi0' / 1='/dev/spi1'> <cs: 0..3> <data length>" },
    { "stdio",   3, 0, (funcPtrUnion)tspi_std_rw,  "<dev_index: 0='/dev/spi0' / 1='/dev/spi1'> <cs: 0..3> <data length>" },
};


int main( int argc, char **argv )
{
    int         i;
    int         rc;

    if ( argc < 2 )
    {
usage:
        printf( "usage:  spitest <cmd> <dev_index> <cs> [<p1> [<p2>...]]\n" );
        printf( "commands:\n" );
        printf( "  iotext <dev_index: 0='/dev/spi0' / 1='/dev/spi1'> <cs: 0..3> <data length> <data_string>\n" );

        for ( i = 0; i < sizeof( commandEntries ) / sizeof( commandEntryType ); ++i )
            printf( "  %s %s\n", commandEntries[i].cmd_name, commandEntries[i].help );
        return (-1);
    }

    for ( i = 0; i < sizeof( commandEntries ) / sizeof( commandEntryType ); ++i )
    {
        if ( !strcmp( argv[1],"iotext" ) )
        {
            if ( argc < 6 )
                goto usage;
            tspi_iodata_test( strtoul( argv[2], NULL, 0 ),
                              strtoul( argv[3], NULL, 0 ),
                              strtoul( argv[4], NULL, 0 ), argv[5] );
            return (0);
        }

        if ( !strcmp( argv[1], commandEntries[i].cmd_name ) )
        {
            if ( commandEntries[i].num_params == 0 )
            {
                if ( argc < 2 )
                    goto usage;
                //if ( argc > 2 )
                //    gpio_debug = 1;
                rc = commandEntries[i].funcPtr.p0FuncPtr();
                if ( commandEntries[i].display_output )
                    printf( "%d\n", rc );
                return (0);
            }

            if ( commandEntries[i].num_params == 1 )
            {
                if ( argc < 3 )
                    goto usage;
                //if ( argc > 3 )
                //    gpio_debug = 1;
                rc = commandEntries[i].funcPtr.p1FuncPtr( strtoul( argv[2], NULL, 0 ) );
                if ( commandEntries[i].display_output )
                    printf( "%d\n", rc );
                return (0);
            }

            if ( commandEntries[i].num_params == 2 )
            {
                if ( argc < 4 )
                    goto usage;
                //if ( argc > 4 )
                //    gpio_debug = 1;
                rc = commandEntries[i].funcPtr.p2FuncPtr( strtoul( argv[2], NULL, 0 ),
                                                          strtoul( argv[3], NULL, 0 ) );
                if ( commandEntries[i].display_output )
                    printf( "%d\n", rc );
                return (0);
            }

            if ( commandEntries[i].num_params == 3 )
            {
                if ( argc < 5 )
                    goto usage;
                //if ( argc > 5 )
                //    gpio_debug = 1;
                rc = commandEntries[i].funcPtr.p3FuncPtr( strtoul( argv[2], NULL, 0 ),
                                                          strtoul( argv[3], NULL, 0 ),
                                                          strtoul( argv[4], NULL, 0 ) );
                if ( commandEntries[i].display_output )
                    printf( "%d\n", rc );
                return (0);
            }

            if ( commandEntries[i].num_params == 4 )
            {
                if ( argc < 6 )
                    goto usage;
                //if ( argc > 6 )
                //    gpio_debug = 1;
                rc = commandEntries[i].funcPtr.p4FuncPtr( strtoul( argv[2], NULL, 0 ),
                                                          strtoul( argv[3], NULL, 0 ),
                                                          strtoul( argv[4], NULL, 0 ),
                                                          strtoul( argv[5], NULL, 0 ) );
                if ( commandEntries[i].display_output )
                    printf( "%d\n", rc );
                return (0);
            }
        }
    }

    goto usage;
}
